@@ -29,8 +29,9 @@ from fridrich import *
 ############################################################################
 #                     global Variable definition                           #
 ############################################################################
-# Protocol version information
-COMM_PROTOCOL_VERSION = "1.1.1"
+# Protocol version and information
+COMM_PROTOCOL_VERSION = "1.1.2"
+CONTROL_CHARACTER: bytes = b"|-|"
 
 # debugging
 DEBUGGER = Debugger(os.getcwd()+"/logs/fridrich.err", file_mode="a")
@@ -249,7 +250,7 @@ class Connection:
 
         return responses
 
-    def _send(self, dictionary: dict, wait: bool = False) -> float | None:
+    def _send(self, dictionary: dict, wait: bool = False, check_auth: bool = True) -> float | None:
         """
         send messages to server
 
@@ -257,17 +258,22 @@ class Connection:
         :param wait: don't send the messages immediately and wait
         :return: time of sending
         """
-        if not self:
-            raise AuthError("Cannot send message - not authenticated yet!")
         self.__message_pool.append(dictionary)
         if not wait:
-            return self.__send()
+            return self.__send(check_auth=check_auth)
 
-    def __send(self) -> float:
+    def __send(self, check_auth: bool = True) -> float:
         """
         the actual sending process
         """
         # check errors before executing
+
+        # we need those calls to happen after each other, so there are 2 ifs
+        if check_auth and not self:
+            print(f"{check_auth=}")
+            if not self.re_auth():
+                raise AuthError("Cannot send message - not authenticated yet!")
+
         if not self.server_ip:
             raise Error("no host set")
 
@@ -288,7 +294,9 @@ class Connection:
 
         if self.__AuthKey:
             string_mes = json.dumps(message, ensure_ascii=True)
-            mes = cryption_tools.MesCryp.encrypt(string_mes, key=self.__AuthKey.encode())
+            # add CONTROL_CHARACTER in front and back for easy checking if the whole message arrived
+            mes = CONTROL_CHARACTER+cryption_tools.MesCryp.encrypt(string_mes,
+                                                                   key=self.__AuthKey.encode())+CONTROL_CHARACTER
 
             try:
                 self.Server.send(mes)
@@ -356,40 +364,49 @@ class Connection:
                 bs = self.Server.recv(8)    # receive message length
                 (length,) = struct.unpack('>Q', bs)
 
+                if self._debug_mode == "full":
+                    print(f"new message, receiving now")
+
+                data = b''
+                no_rec = 0
+                to_read = 0
+                while len(data) < length:  # receive message in patches so size doesn't matter
+                    o_to_read = to_read
+                    to_read = length - len(data)
+                    data += self.Server.recv(
+                        4096 if to_read > 4096 else to_read
+                    )
+
+                    if to_read == o_to_read:  # check if new packages were received
+                        no_rec += 1
+                    else:
+                        no_rec = 0
+
+                    if no_rec >= 100:  # if for 100 loops no packages were received, raise connection loss
+                        raise socket.error('Failed receiving data - connection loss')
+
             except (ConnectionResetError, struct.error, socket.timeout):
                 continue
-            if self._debug_mode == "full":
-                print(f"new message, receiving now")
-
-            data = b''
-            no_rec = 0
-            to_read = 0
-            while len(data) < length:   # receive message in patches so size doesn't matter
-                o_to_read = to_read
-                to_read = length - len(data)
-                data += self.Server.recv(
-                                    4096 if to_read > 4096 else to_read
-                                    )
-
-                if to_read == o_to_read:    # check if new packages were received
-                    no_rec += 1
-                else:
-                    no_rec = 0
-
-                if no_rec >= 100:          # if for 100 loops no packages were received, raise connection loss
-                    raise socket.error('Failed receiving data - connection loss')
 
             if self._debug_mode == "full":
                 print("received data")
 
+            if not data.startswith(CONTROL_CHARACTER) or not data.endswith(CONTROL_CHARACTER):
+                raise MessageError("Message got lost - only a part arrived")
+
+            if self._debug_mode == "full":
+                print(f"starts and ends correctly, \"{data[:5]}\" : \"{data[-5::]}\"")
+
             try:
-                mes = cryption_tools.MesCryp.decrypt(data, self.__AuthKey.encode())
-            except cryption_tools.InvalidToken:
-                self._messages["Error"] = {"Error": "MessageError", "info": f"cant decrypt: {data}"}
+                cl = len(CONTROL_CHARACTER)
+                mes = cryption_tools.MesCryp.decrypt(data[cl:-cl], self.__AuthKey.encode())
+
+            except (cryption_tools.InvalidToken, AttributeError):
+                self._messages["Error"] = {"Error": "MessageError", "info": f"cant decrypt: \"{data[:5]}\" : \"{data[-5::]}\""}
                 continue
 
             if self._debug_mode in ("full", "normal"):
-                print(f"decrypted data: {mes=}")
+                print(f"decrypted data: {mes}")
 
             try:
                 for _ in range(2):
@@ -414,8 +431,8 @@ class Connection:
                             self._messages["Error"] = message["content"]
 
                         case "disconnect":
-                            self._messages["disconnect"] = True
-                            self.end()
+                            print(f"Server disconnect! reason: {message['content']['reason']}")
+                            self.end(revive=True)
 
                         case "ServerRequest":
                             self._server_messages[mes['time']] = message
@@ -541,6 +558,9 @@ class Connection:
         """
         if a user has already been logged in, you can now try to re-login
         """
+        if self._debug_mode in ("normal", "full"):
+            print("re-authing")
+
         if self.__userN is None or self.__pwd_hash is None:
             return False
 
@@ -951,7 +971,8 @@ class Connection:
         return self.del_var(item)
 
     # WeatherStation Funcs
-    def register_station(self, station_name: str, location: str, wait: bool = False) -> Union[FridrichFuture, None]:
+    def register_station(self, station_name: str, location: str,
+                         wait: bool = False, **station_info) -> Union[FridrichFuture, None]:
         """
         register a new weather station
         """
@@ -960,6 +981,7 @@ class Connection:
             "station_name": station_name,
             "location": location
         }
+        msg.update(station_info)
         self._send(msg, wait=True)
 
         # result handling
@@ -1313,7 +1335,10 @@ class Connection:
         """
         return True if AuthKey
         """
-        return bool(self.__AuthKey)
+        try:
+            return bool(self.__AuthKey)
+        except (Exception,):
+            return False
 
     def __enter__(self) -> "Connection":
         return self
@@ -1324,9 +1349,6 @@ class Connection:
             return False
         return True
 
-    def __del__(self) -> None:
-        self.end(revive=False)
-
     def __eq__(self, other: "Connection") -> bool:
         if not type(other) == Connection:
             return self.__bool__()
@@ -1343,7 +1365,7 @@ class Connection:
                'type': 'end'
         }    # set message
         with suppress(ConnectionResetError, ConnectionAbortedError, AuthError):
-            self._send(msg, wait=False)  # send message
+            self._send(msg, wait=False, check_auth=False)  # send message
 
         self.loop = False
         self.__AuthKey = None
